@@ -2,6 +2,6 @@ from django.urls import re_path
 
 from notification.consumers import NotificationConsumer
 
-ws_routers = [
+wspatterns = [
     re_path(r"^notify/message/$", NotificationConsumer.as_asgi()),
 ]
